var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect, useState, useRef } from "react";
import { isFunction, isNil, isArray } from "lodash";
import CSS from "primeobjects-ui-web-store/build/cjs/css";
import { isNonEmptyString, isSSR } from "primeobjects-helper-util/build/cjs/core";
import { newGuid } from "primeobjects-helper-util/build/cjs/uuid";
import { useEditor, EditorContent, ReactNodeViewRenderer } from "@tiptap/react";
import Typography from "@tiptap/extension-typography";
import Link from "@tiptap/extension-link";
import Image from "@tiptap/extension-image";
import Dropcursor from "@tiptap/extension-dropcursor";
import StarterKit from "@tiptap/starter-kit";
import CodeBlockLowlight from "@tiptap/extension-code-block-lowlight";
import HtmlEditorCode from "./html-code";
import Highlight from "@tiptap/extension-highlight";
import Color from "@tiptap/extension-color";
import { HTML_EDITOR_CSS } from "./html-css";
import { HTML_EDITOR_CODE_CSS } from "./html-code-css";
import lowlight from "lowlight";
import HtmlEditorFloatEditToolbar from "./html-float-edit-toolbar";
import HtmlEditorFloatInsertToolbar from "./html-float-insert-toolbar";
import HtmlEditorFloatHighlightToolbar from "./html-float-highlight-toolbar";
// load specific languages only
// import lowlight from 'lowlight/lib/core'
// import javascript from 'highlight.js/lib/languages/javascript'
// lowlight.registerLanguage('javascript', javascript)
export var HtmlEditor = function (props) {
    var highlightCustomButtons = props.highlightCustomButtons, onHighlight = props.onHighlight;
    var placeholder = isNonEmptyString(props.placeholder) ? props.placeholder : "";
    var _a = useState(isNonEmptyString(props.content) ? props.content : ""), content = _a[0], setContent = _a[1];
    var id = useState(isNonEmptyString(props.id) ? props.id : newGuid())[0];
    var _b = useState(!isNil(props.recordId) && isNonEmptyString(props.recordId) ? props.recordId : newGuid()), recordId = _b[0], setRecordId = _b[1];
    var className = props.className ? props.className : "";
    var style = props.style ? props.style : {};
    var inputClassName = props.inputClassName ? props.inputClassName : "";
    var inputStyle = props.inputStyle ? props.inputStyle : {};
    var readonly = useRef(props.readonly == true);
    var hideH1 = props.hideH1 == true;
    var hideH2 = props.hideH2 == true;
    var hideH3 = props.hideH3 == true;
    var hideH4 = props.hideH4 == true;
    var supportSourceCode = props.supportSourceCode == true;
    var supportHighlight = props.supportHighlight == true;
    var entityName = !isNil(props.entityName) && isNonEmptyString(props.entityName) ? props.entityName : "HTMLEditor";
    var attributeName = !isNil(props.attributeName) && isNonEmptyString(props.attributeName) ? props.attributeName : "content";
    var highlightColors = !isNil(props.highlightColors) && isArray(props.highlightColors) ? props.highlightColors : [];
    useEffect(function () {
        if (!isNil(props.recordId) && isNonEmptyString(props.recordId)) {
            setRecordId(props.recordId);
        }
    }, [props.recordId]);
    useEffect(function () {
        readonly.current = props.readonly == true;
        onInit();
    }, [props.readonly]);
    useEffect(function () {
        var newContent = isNonEmptyString(props.content) ? props.content : "";
        setContent(newContent);
    }, [props.content]);
    var setPrevContent = function (content) {
        var o = document.getElementById("html-for-prev-".concat(id));
        if (o)
            o.innerHTML = content;
    };
    var getPrevContent = function () {
        var o = document.getElementById("html-for-prev-".concat(id));
        return o ? o.innerHTML : "";
    };
    var onInit = function () {
        var htmlEditor = document.getElementsByClassName("html-editor-".concat(id));
        htmlEditor = htmlEditor.length > 0 && htmlEditor[0];
        var editorDivs = htmlEditor === null || htmlEditor === void 0 ? void 0 : htmlEditor.getElementsByClassName("ProseMirror");
        if (editorDivs && editorDivs.length > 0) {
            if (readonly.current) {
                editorDivs[0].setAttribute("contenteditable", "false");
            }
            else {
                editorDivs[0].setAttribute("contenteditable", "true");
            }
            //remove empty br
            var fieldEmptyBr = htmlEditor.getElementsByClassName("ProseMirror-trailingBreak");
            for (var i = 0; i < fieldEmptyBr.length; i++) {
                if (fieldEmptyBr[i].tagName == "BR" && fieldEmptyBr[i].parentElement.children.length == 1) {
                    fieldEmptyBr[i].parentElement.remove();
                }
            }
        }
    };
    var onEvent = function (editor, action, info) {
        var _a, _b, _c, _d;
        var html = editor.getHTML();
        var htmlEditor = document.getElementsByClassName("html-editor-".concat(id));
        htmlEditor = htmlEditor.length > 0 && htmlEditor[0];
        if (action == "init") {
            setTimeout(function () {
                setPrevContent(html);
                onInit();
                if (isFunction(props.onReady))
                    props.onReady();
            }, 200);
        }
        if ((action == "init" || action == "blur") && (html == "<p></p>" || html == "<p></p><p></p>")) {
            if (htmlEditor)
                htmlEditor.className = "".concat(htmlEditor.className, " is-placeholder html-editor-").concat(id, "-is-placeholder");
            (_a = editor === null || editor === void 0 ? void 0 : editor.commands) === null || _a === void 0 ? void 0 : _a.setContent("<p>".concat(placeholder, "</p>"));
        }
        if (action == "focus" && html == "<p>".concat(placeholder, "</p>")) {
            if (htmlEditor)
                htmlEditor.className = htmlEditor.className.replace("html-editor-".concat(id, "-is-placeholder"), "").replace("is-placeholder", "").trim();
            (_b = editor === null || editor === void 0 ? void 0 : editor.commands) === null || _b === void 0 ? void 0 : _b.setContent("<p></p><p></p>");
        }
        if ((action == "selection" || action == "focus") && htmlEditor) {
            if (html == "<p></p>") {
                if (htmlEditor.className.indexOf("is-empty") < 0)
                    htmlEditor.className = "".concat(htmlEditor.className, " is-empty");
            }
            else {
                if (htmlEditor.className.indexOf("is-empty") > 0)
                    htmlEditor.className = htmlEditor.className.replace("is-empty", "");
            }
        }
        if (action == "update") {
            if (htmlEditor)
                htmlEditor.className = htmlEditor.className.replace("html-editor-".concat(id, "-is-placeholder"), "").replace("is-placeholder", "").trim();
            if (readonly.current && (info === null || info === void 0 ? void 0 : info.type) == "paste") {
                //rollback
                (_c = editor === null || editor === void 0 ? void 0 : editor.commands) === null || _c === void 0 ? void 0 : _c.setContent(getPrevContent());
            }
            (_d = document.getElementById("html-for-sync-".concat(id))) === null || _d === void 0 ? void 0 : _d.click();
        }
    };
    var editor = useEditor({
        content: content,
        extensions: [
            StarterKit,
            Highlight.configure({ multicolor: true }),
            Color,
            Link.configure({
                openOnClick: false
            }),
            Image,
            Dropcursor,
            Typography,
            CodeBlockLowlight.extend({
                addNodeView: function () {
                    return ReactNodeViewRenderer(function (props) {
                        return React.createElement(HtmlEditorCode, __assign({}, props, { readonly: readonly.current }));
                    });
                }
            }).configure({ lowlight: lowlight })
        ],
        onBeforeCreate: function () {
            //const { editor } = createParams;
        },
        onCreate: function (_a) {
            var _b;
            var editor = _a.editor;
            //the line below has to be done here, do not move inside onEvent
            (_b = document.getElementById("html-for-init-".concat(id))) === null || _b === void 0 ? void 0 : _b.click();
            onEvent(editor, "init");
        },
        onUpdate: function (updateParams) {
            var _a;
            var editor = updateParams.editor;
            var transaction = updateParams.transaction;
            //const transaction: Record<string, any> = updateParams?.transaction;
            onEvent(editor, "update", { type: ((_a = transaction === null || transaction === void 0 ? void 0 : transaction.meta) === null || _a === void 0 ? void 0 : _a.paste) == true ? "paste" : "default" });
        },
        onSelectionUpdate: function (_a) {
            var editor = _a.editor;
            onEvent(editor, "selection");
            // editor.chain().focus().setImage({ src: url }).run()
            // The selection has changed.
        },
        onTransaction: function (_a) {
            var editor = _a.editor;
            onEvent(editor, "transaction");
            // The editor state has changed.
        },
        onFocus: function (_a) {
            var editor = _a.editor;
            onEvent(editor, "focus");
        },
        onBlur: function (_a) {
            var editor = _a.editor;
            onEvent(editor, "blur");
        },
        onDestroy: function () {
            // The editor is being destroyed.
        }
    }, [recordId]);
    var onInitContent = function () {
        var _a;
        var curValue = editor === null || editor === void 0 ? void 0 : editor.getHTML();
        if (curValue != content) {
            (_a = editor === null || editor === void 0 ? void 0 : editor.commands) === null || _a === void 0 ? void 0 : _a.setContent(content);
        }
    };
    var onSyncContent = function () {
        var newContent = editor === null || editor === void 0 ? void 0 : editor.getHTML();
        if (newContent == "" || newContent == "<p></p>")
            newContent = "<p></p><p></p>";
        setPrevContent(newContent);
        setContent(newContent);
        if (isFunction(props.onChange))
            props.onChange(isNonEmptyString(newContent) ? newContent : "");
    };
    return isSSR() ? (React.createElement(React.Fragment, null)) : (React.createElement(React.Fragment, null,
        React.createElement(CSS, { id: "html-editor-css", content: HTML_EDITOR_CSS }),
        React.createElement(CSS, { id: "html-editor-code-css", content: HTML_EDITOR_CODE_CSS }),
        React.createElement("div", { className: "w-full html-editor html-editor-".concat(id, " ").concat(readonly.current == true ? "readonly-true" : "readonly-false", " ").concat(className), style: style },
            editor && React.createElement(HtmlEditorFloatInsertToolbar, { recordId: recordId, entityName: entityName, attributeName: attributeName, editor: editor }),
            editor && (React.createElement(HtmlEditorFloatEditToolbar, { editor: editor, hideH1: hideH1, hideH2: hideH2, hideH3: hideH3, hideH4: hideH4, supportHighlight: supportHighlight, supportSourceCode: supportSourceCode })),
            editor && supportHighlight && React.createElement(HtmlEditorFloatHighlightToolbar, { editor: editor, colors: highlightColors, customButtons: highlightCustomButtons, onHighlight: onHighlight }),
            React.createElement(EditorContent, { editor: editor, className: "editor ".concat(readonly.current == true ? "readonly-true" : "readonly-false", " ").concat(inputClassName), style: inputStyle }),
            React.createElement("div", { id: "html-for-init-".concat(id), onClick: onInitContent }),
            React.createElement("div", { id: "html-for-prev-".concat(id), className: "hidden" }),
            React.createElement("div", { id: "html-for-sync-".concat(id), onClick: onSyncContent }))));
};
export default HtmlEditor;
